/*
 * Please do not edit this file.
 * It was generated using rpcgen.
 */

#ifndef _RNUSERS_H_RPCGEN
#define	_RNUSERS_H_RPCGEN

#include <rpc/rpc.h>

#ifdef __cplusplus
extern "C" {
#endif

#define	MAXUSERS 100
#define	MAXUTLEN 256

struct utmp {
	char *ut_line;
	char *ut_name;
	char *ut_host;
	int ut_time;
};
typedef struct utmp utmp;

struct utmpidle {
	utmp ui_utmp;
	u_int ui_idle;
};
typedef struct utmpidle utmpidle;

typedef struct {
	u_int utmparr_len;
	utmp *utmparr_val;
} utmparr;

typedef struct {
	u_int utmpidlearr_len;
	utmpidle *utmpidlearr_val;
} utmpidlearr;
#define	RUSERS_MAXUSERLEN 32
#define	RUSERS_MAXLINELEN 32
#define	RUSERS_MAXHOSTLEN 257

struct rusers_utmp {
	char *ut_user;
	char *ut_line;
	char *ut_host;
	int ut_type;
	int ut_time;
	u_int ut_idle;
};
typedef struct rusers_utmp rusers_utmp;

typedef struct {
	u_int utmp_array_len;
	rusers_utmp *utmp_array_val;
} utmp_array;

#define	RUSERSPROG ((unsigned long)(100002))
#define	RUSERSVERS_ORIG ((unsigned long)(1))

extern  void rusersprog_1(struct svc_req *rqstp, SVCXPRT *transp);
#define	RUSERSPROC_NUM ((unsigned long)(1))
extern  int * rusersproc_num_1(void *, CLIENT *);
extern  int * rusersproc_num_1_svc(void *, struct svc_req *);
#define	RUSERSPROC_NAMES ((unsigned long)(2))
extern  utmparr * rusersproc_names_1(void *, CLIENT *);
extern  utmparr * rusersproc_names_1_svc(void *, struct svc_req *);
#define	RUSERSPROC_ALLNAMES ((unsigned long)(3))
extern  utmparr * rusersproc_allnames_1(void *, CLIENT *);
extern  utmparr * rusersproc_allnames_1_svc(void *, struct svc_req *);
extern int rusersprog_1_freeresult(SVCXPRT *, xdrproc_t, caddr_t);
#define	RUSERSVERS_IDLE ((unsigned long)(2))

extern  void rusersprog_2(struct svc_req *rqstp, SVCXPRT *transp);
extern  int * rusersproc_num_2(void *, CLIENT *);
extern  int * rusersproc_num_2_svc(void *, struct svc_req *);
extern  utmpidlearr * rusersproc_names_2(void *, CLIENT *);
extern  utmpidlearr * rusersproc_names_2_svc(void *, struct svc_req *);
extern  utmpidlearr * rusersproc_allnames_2(void *, CLIENT *);
extern  utmpidlearr * rusersproc_allnames_2_svc(void *, struct svc_req *);
extern int rusersprog_2_freeresult(SVCXPRT *, xdrproc_t, caddr_t);
#define	RUSERSVERS_3 ((unsigned long)(3))

extern  void rusersprog_3(struct svc_req *rqstp, SVCXPRT *transp);
extern  int * rusersproc_num_3(void *, CLIENT *);
extern  int * rusersproc_num_3_svc(void *, struct svc_req *);
extern  utmp_array * rusersproc_names_3(void *, CLIENT *);
extern  utmp_array * rusersproc_names_3_svc(void *, struct svc_req *);
extern  utmp_array * rusersproc_allnames_3(void *, CLIENT *);
extern  utmp_array * rusersproc_allnames_3_svc(void *, struct svc_req *);
extern int rusersprog_3_freeresult(SVCXPRT *, xdrproc_t, caddr_t);

/* the xdr functions */
extern  bool_t xdr_utmp(XDR *, utmp*);
extern  bool_t xdr_utmpidle(XDR *, utmpidle*);
extern  bool_t xdr_utmparr(XDR *, utmparr*);
extern  bool_t xdr_utmpidlearr(XDR *, utmpidlearr*);
extern  bool_t xdr_rusers_utmp(XDR *, rusers_utmp*);
extern  bool_t xdr_utmp_array(XDR *, utmp_array*);

#ifdef __cplusplus
}
#endif

#endif /* !_RNUSERS_H_RPCGEN */
