/*
 * Please do not edit this file.
 * It was generated using rpcgen.
 */

#ifndef _YPPASSWD_H_RPCGEN
#define	_YPPASSWD_H_RPCGEN

#include <rpc/rpc.h>

#ifdef __cplusplus
extern "C" {
#endif


struct x_passwd {
	char *pw_name;
	char *pw_passwd;
	int pw_uid;
	int pw_gid;
	char *pw_gecos;
	char *pw_dir;
	char *pw_shell;
};
typedef struct x_passwd x_passwd;

struct yppasswd {
	char *oldpass;
	x_passwd newpw;
};
typedef struct yppasswd yppasswd;
#include <sys/cdefs.h>
extern int _yppasswd( char * , struct x_passwd * );
#define yppasswd(x,y) _yppasswd(x,y)

#define	YPPASSWDPROG ((unsigned long)(100009))
#define	YPPASSWDVERS ((unsigned long)(1))

extern  void yppasswdprog_1(struct svc_req *rqstp, SVCXPRT *transp);
#define	YPPASSWDPROC_UPDATE ((unsigned long)(1))
extern  int * yppasswdproc_update_1(yppasswd *, CLIENT *);
extern  int * yppasswdproc_update_1_svc(yppasswd *, struct svc_req *);
extern int yppasswdprog_1_freeresult(SVCXPRT *, xdrproc_t, caddr_t);

/* the xdr functions */
extern  bool_t xdr_x_passwd(XDR *, x_passwd*);
extern  bool_t xdr_yppasswd(XDR *, yppasswd*);

#ifdef __cplusplus
}
#endif

#endif /* !_YPPASSWD_H_RPCGEN */
