/*-
 * SPDX-License-Identifier: BSD-2-Clause
 *
 * Copyright (c) 2024 Mark Johnston <markj@FreeBSD.org>
 */

#ifndef _SYS_SDT_MACHDEP_H_
#define	_SYS_SDT_MACHDEP_H_

#define	_SDT_ASM_PATCH_INSTR	"nopw 0(%%rax,%%rax,1)"

#endif
