/*
 * Please do not edit this file.
 * It was generated using rpcgen.
 */

#ifndef _SPRAY_H_RPCGEN
#define	_SPRAY_H_RPCGEN

#include <rpc/rpc.h>

#ifdef __cplusplus
extern "C" {
#endif

#define	SPRAYMAX 8845

struct spraytimeval {
	u_int sec;
	u_int usec;
};
typedef struct spraytimeval spraytimeval;

struct spraycumul {
	u_int counter;
	spraytimeval clock;
};
typedef struct spraycumul spraycumul;

typedef struct {
	u_int sprayarr_len;
	char *sprayarr_val;
} sprayarr;

#define	SPRAYPROG ((unsigned long)(100012))
#define	SPRAYVERS ((unsigned long)(1))

extern  void sprayprog_1(struct svc_req *rqstp, SVCXPRT *transp);
#define	SPRAYPROC_SPRAY ((unsigned long)(1))
extern  void * sprayproc_spray_1(sprayarr *, CLIENT *);
extern  void * sprayproc_spray_1_svc(sprayarr *, struct svc_req *);
#define	SPRAYPROC_GET ((unsigned long)(2))
extern  spraycumul * sprayproc_get_1(void *, CLIENT *);
extern  spraycumul * sprayproc_get_1_svc(void *, struct svc_req *);
#define	SPRAYPROC_CLEAR ((unsigned long)(3))
extern  void * sprayproc_clear_1(void *, CLIENT *);
extern  void * sprayproc_clear_1_svc(void *, struct svc_req *);
extern int sprayprog_1_freeresult(SVCXPRT *, xdrproc_t, caddr_t);

/* the xdr functions */
extern  bool_t xdr_spraytimeval(XDR *, spraytimeval*);
extern  bool_t xdr_spraycumul(XDR *, spraycumul*);
extern  bool_t xdr_sprayarr(XDR *, sprayarr*);

#ifdef __cplusplus
}
#endif

#endif /* !_SPRAY_H_RPCGEN */
