/*-
 * Copyright (c) 1992-2026 The FreeBSD Project.
 *
 */
#ifdef _KERNEL
#error "<osreldate.h> cannot be used in the kernel, use <sys/param.h>"
#else
#undef __FreeBSD_version
#define __FreeBSD_version 1500068
#endif
