// -*- C++ -*-
//===----------------------------------------------------------------------===//
//
// Part of the LLVM Project, under the Apache License v2.0 with LLVM Exceptions.
// See https://llvm.org/LICENSE.txt for license information.
// SPDX-License-Identifier: Apache-2.0 WITH LLVM-exception
//
//===----------------------------------------------------------------------===//

export {
  using ::sig_atomic_t _LIBCPP_USING_IF_EXISTS;

  // [support.signal], signal handlers
  using ::signal _LIBCPP_USING_IF_EXISTS;

  using ::raise _LIBCPP_USING_IF_EXISTS;
} // export
