
/*
 * Copyright (C) 2012 by Darren Reed.
 *
 * See the IPFILTER.LICENCE file for details on licencing.
 * Id: ipl.h,v 2.52.2.30 2007/10/16 09:41:00 darrenr Exp $
 */

#ifndef	__IPL_H__
#define	__IPL_H__

#define	IPL_VERSION	"IP Filter: v5.1.2"

#define	IPFILTER_VERSION	5010200

#endif	/* __IPL_H__ */
