/*-
 * This file is in the public domain.
 */

#ifndef	_MACHINE_IOMMU_H_
#define	_MACHINE_IOMMU_H_

#include <x86/include/busdma_impl.h>
#include <x86/iommu/intel_reg.h>
#include <x86/iommu/x86_iommu.h>
#include <x86/iommu/intel_dmar.h>

#endif /* !_MACHINE_IOMMU_H_ */
