/*
 * Please do not edit this file.
 * It was generated using rpcgen.
 */

#ifndef _YPUPDATE_PROT_H_RPCGEN
#define	_YPUPDATE_PROT_H_RPCGEN

#include <rpc/rpc.h>

#ifdef __cplusplus
extern "C" {
#endif

/*
 * Sun RPC is a product of Sun Microsystems, Inc. and is provided for
 * unrestricted use provided that this legend is included on all tape
 * media and as a part of the software program in whole or part.  Users
 * may copy or modify Sun RPC without charge, but are not authorized
 * to license or distribute it to anyone else except as part of a product or
 * program developed by the user or with the express written consent of
 * Sun Microsystems, Inc.
 *
 * SUN RPC IS PROVIDED AS IS WITH NO WARRANTIES OF ANY KIND INCLUDING THE
 * WARRANTIES OF DESIGN, MERCHANTIBILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE, OR ARISING FROM A COURSE OF DEALING, USAGE OR TRADE PRACTICE.
 *
 * Sun RPC is provided with no support and without any obligation on the
 * part of Sun Microsystems, Inc. to assist in its use, correction,
 * modification or enhancement.
 *
 * SUN MICROSYSTEMS, INC. SHALL HAVE NO LIABILITY WITH RESPECT TO THE
 * INFRINGEMENT OF COPYRIGHTS, TRADE SECRETS OR ANY PATENTS BY SUN RPC
 * OR ANY PART THEREOF.
 *
 * In no event will Sun Microsystems, Inc. be liable for any lost revenue
 * or profits or other special, indirect and consequential damages, even if
 * Sun has been advised of the possibility of such damages.
 *
 * Sun Microsystems, Inc.
 * 2550 Garcia Avenue
 * Mountain View, California  94043
 */
/*
 * Copyright (c) 1986, 1990 by Sun Microsystems, Inc.
 */


/*
 * Compiled from ypupdate_prot.x using rpcgen
 * This is NOT source code!
 * DO NOT EDIT THIS FILE!
 */

#ifndef _rpcsvc_ypupdate_prot_h
#define _rpcsvc_ypupdate_prot_h

#define	MAXMAPNAMELEN 255
#define	MAXYPDATALEN 1023
#define	MAXERRMSGLEN 255

typedef struct {
	u_int yp_buf_len;
	char *yp_buf_val;
} yp_buf;

struct ypupdate_args {
	char *mapname;
	yp_buf key;
	yp_buf datum;
};
typedef struct ypupdate_args ypupdate_args;

struct ypdelete_args {
	char *mapname;
	yp_buf key;
};
typedef struct ypdelete_args ypdelete_args;

#endif /* !_rpcsvc_ypupdate_prot_h */

#define	YPU_PROG ((unsigned long)(100028))
#define	YPU_VERS ((unsigned long)(1))

extern  void ypu_prog_1(struct svc_req *rqstp, SVCXPRT *transp);
#define	YPU_CHANGE ((unsigned long)(1))
extern  u_int * ypu_change_1(ypupdate_args *, CLIENT *);
extern  u_int * ypu_change_1_svc(ypupdate_args *, struct svc_req *);
#define	YPU_INSERT ((unsigned long)(2))
extern  u_int * ypu_insert_1(ypupdate_args *, CLIENT *);
extern  u_int * ypu_insert_1_svc(ypupdate_args *, struct svc_req *);
#define	YPU_DELETE ((unsigned long)(3))
extern  u_int * ypu_delete_1(ypdelete_args *, CLIENT *);
extern  u_int * ypu_delete_1_svc(ypdelete_args *, struct svc_req *);
#define	YPU_STORE ((unsigned long)(4))
extern  u_int * ypu_store_1(ypupdate_args *, CLIENT *);
extern  u_int * ypu_store_1_svc(ypupdate_args *, struct svc_req *);
extern int ypu_prog_1_freeresult(SVCXPRT *, xdrproc_t, caddr_t);

/* the xdr functions */
extern  bool_t xdr_yp_buf(XDR *, yp_buf*);
extern  bool_t xdr_ypupdate_args(XDR *, ypupdate_args*);
extern  bool_t xdr_ypdelete_args(XDR *, ypdelete_args*);

#ifdef __cplusplus
}
#endif

#endif /* !_YPUPDATE_PROT_H_RPCGEN */
